//: # URLs in the macOS file system:
import Cocoa

//: Make sure this playground is called **Files.playground** and that it is on your **Desktop**.
//: ### Explicit file path converted to URL object.

let completePath = "/Users/sarah/Desktop/Files.playground"
let completeUrl = URL(fileURLWithPath: completePath)
//: Not much use unlesss your user name is 'sarah'!


//: ### Create a URL by combining the home directory and the relative path.

let home = FileManager.default.homeDirectoryForCurrentUser
let playgroundPath = "Desktop/Files.playground"
let playgroundUrl = home.appendingPathComponent(playgroundPath)

//: Check out the ways to access parts of the URL:
playgroundUrl.path
playgroundUrl.absoluteString
playgroundUrl.absoluteURL
playgroundUrl.baseURL

playgroundUrl.pathComponents
playgroundUrl.lastPathComponent
playgroundUrl.pathExtension

playgroundUrl.isFileURL
playgroundUrl.hasDirectoryPath
//: Why is it marked as a directory? Because .playground files are really folder bundles like .app files. Right-click on the playground file and select **Show Package Contents** to see what is inside.


//: ### Create a URL by sending a relative path and a base path to the URL initializer.

let relativeUrl = URL(fileURLWithPath: playgroundPath, relativeTo: home)

//: #### See that the parts of the URL are identical to the URL created the first time, except it has a baseURL.
relativeUrl.path
relativeUrl.absoluteString
relativeUrl.absoluteURL
relativeUrl.baseURL

relativeUrl.pathComponents
relativeUrl.lastPathComponent
relativeUrl.pathExtension

relativeUrl.isFileURL
relativeUrl.hasDirectoryPath

//: ### URLs are also used for web addresses but they are not the same!

let webAddress = "https://www.raywenderlich.com"
let webUrl = URL(string: webAddress)
webUrl?.isFileURL

//: ### It is easy to edit URLs:

var urlForEditing = home
urlForEditing.path

urlForEditing.appendPathComponent("Desktop")
urlForEditing.path

urlForEditing.appendPathComponent("Test file")
urlForEditing.path

urlForEditing.appendPathExtension("txt")
urlForEditing.path

urlForEditing.deletePathExtension()
urlForEditing.path

urlForEditing.deleteLastPathComponent()
urlForEditing.path

//: ### Or to create new ones based on existing URLs:
let fileUrl = home
    .appendingPathComponent("Desktop")
    .appendingPathComponent("Test file")
    .appendingPathExtension("txt")
fileUrl.path

let desktopUrl = fileUrl.deletingLastPathComponent()
desktopUrl.path

//: ### To check whether a file exists, you have to use a string representation of the file path.

let fileManager = FileManager.default
fileManager.fileExists(atPath: playgroundUrl.path)

let missingFile = URL(fileURLWithPath: "this_file_does_not_exist.missing")
fileManager.fileExists(atPath: missingFile.path)

//: ### And here is the way to check if it is a folder.

var isDirectory: ObjCBool = false
fileManager.fileExists(atPath: relativeUrl.path, isDirectory: &isDirectory)
isDirectory.boolValue
